<?php
namespace Service;

use Zeedhi\Framework\Session\Session;
use Zeedhi\Framework\DataSource\FilterCriteria;
use Zeedhi\Framework\DataSource\DataSet;
use \Service\Structure;
use \Service\Manager\Mongo\Manager;
use \Service\ViewProperty;
use Zeedhi\Zmart\DataSource\ZmartFilterCriteria;
use \Util\Utilities;

class ImportHistory {

    protected $dataSourceName = 'importHistory';
    /** @var \Service\Manager\Mongo\Manager $mongoManager */
    protected $mongoManager;
    /** @var Session $sessionManager */
    protected $sessionManager;
    /** @var Utilities $utilities */
    protected $utilities;

    public function __construct(Manager $mongoManager, Session $sessionManager, Utilities $utilities) {
        $this->mongoManager = $mongoManager;
        $this->sessionManager = $sessionManager;
        $this->utilities = $utilities;
    }

    public function findBy(FilterCriteria $filterCriteria) {
        return $this->mongoManager->findBy($filterCriteria)->getRows();
    }

    public function findAll() {
        return $this->findBy(new FilterCriteria($this->dataSourceName));
    }

    public function findById($id) {
        return $this->findByColumn('_id', $id);
    }

    public function findByDataSourceId($dataSourceId) {
        return $this->findByColumn('dataSourceId', $dataSourceId);
    }

    public function findByColumn($column, $value){
        $filterCriteria = new FilterCriteria($this->dataSourceName, array(
            array(
                "columnName"=>$column,
                "value" => $value,
                "operator" => "="
            )
        ));
        $importHistory = $this->mongoManager->findBy($filterCriteria)->getRows();
        return $importHistory;
    }

    public function findByProcessId($processId) {
        return $this->findByColumn('process', $processId);
    }

    public function findImportsInProgressBlocked($dataSourceId){
        $filterCriteria = new FilterCriteria($this->dataSourceName);
        $filterCriteria->addCondition('dataSourceId', $dataSourceId);
        $filterCriteria->addCondition('importStatus', 'blocked');
        $historysBlockeds = $this->findBy($filterCriteria);

        return $historysBlockeds;
    }

    public function findImportsInProgress($dataSourceId){
        $historys = [];
        $importHistory = $this->findByDataSourceId($dataSourceId);

        foreach ($importHistory as $history) {
            $importArrivedAtTheEnd = $history['endDate'] === null && $history['elapsedTime'] === null;
            $importOfSubprocessesArrivedAtTheEnd = $history['subprocesses'] >= $history['imported'] && !$history['hasError'];

            $importInProgress = false;
            if(isset($history['importStatus'])){
                $conditions = ["start", "importing", "pending", "queue"];
                $importInProgress = in_array($history['importStatus'], $conditions);
            }

            if($importArrivedAtTheEnd && $importInProgress && $importOfSubprocessesArrivedAtTheEnd){
                array_push($historys, $history);
            }
        }

        return $historys;
    }

    public function findLastOnesByDataSourceIds($dataSourceIds, $date = null) {
        $importData = array();
        $filterCriteria = new FilterCriteria($this->dataSourceName);
        $filterCriteria->addCondition('organizationId', (string)\Util\Utilities::getOrganizationIdForMongo());
        $filterCriteria->addCondition('dataSourceId', FilterCriteria::IN, $dataSourceIds);
        $filterCriteria->addOrderBy('createDate', FilterCriteria::ORDER_DESC);

        if ($date) {
            $filterCriteria->addCondition('createDate', $date);
        }

        $importData = $this->mongoManager->findBy($filterCriteria)->getRows();
        if (!$date) {
            $importData = $this->filterImportHistoryByLastDate($importData);
        }
        return $importData;
    }

    protected function filterImportHistoryByLastDate($importData) {
        $newImportData = array();

        if (!empty($importData)) {
            $usedItems = array($importData[0]['dataSourceId'] => $importData[0]['createDate']);

            foreach ($importData as $index => $import) {
                $dataSourceId = $import['dataSourceId'];
                $start = $import['createDate'];
                if (isset($usedItems[$dataSourceId])) {
                    if ($start === $usedItems[$dataSourceId]) {
                        $newImportData[] = $import;
                    }
                } else {
                    $newImportData[] = $import;
                    $usedItems[$dataSourceId] = $start;
                }
            }
        }
        return $newImportData;
    }

    public function prepareHistoryRows(&$rows) {
        $rowsIndexedByDataSourceId = array();

        foreach ($rows as &$row) {

            if (isset($row['elapsedTime']) && $row['elapsedTime']) {
                $row['elapsedTime'] = gmdate("H:i:s", $row['elapsedTime']);
            }

            $status = isset($row['importStatus']) ? $row['importStatus'] : null;

            if ($status === 'canceled') {
                $row['status'] = 'c';
            } else if ($status === 'saving') {
                $row['status'] = 's';
            } else if (isset($row['subprocesses']) && isset($row['imported']) && $row['subprocesses'] > 0) {
                $row['status'] = (round(100 * $row['imported'] / $row['subprocesses'], 0));
            } else if (!empty($row['endDate'])) {
                $row['status'] = 100;
            } else {
                $row['status'] = 0;
            }
            $row['createDate'] = $this->formatDate($row['createDate']);
            $row['endDate'] = $this->formatDate($row['endDate']);
            $rowsIndexedByDataSourceId[$row['dataSourceId']] = $row;
        }
        return $rowsIndexedByDataSourceId;
    }

    private function formatDate ($dateTime){
        if (!empty($dateTime) && $dateTime instanceof \DateTime) {
            $dateTime->setTimezone(new \DateTimeZone('America/Sao_Paulo'));
            $dateTime = $dateTime->format("d/m/Y H:i");
        }
        return $dateTime;
    }

    public function getLastImportationDate($dataSourceId){
        $lastImportDate = null;
        $filterCriteria = new FilterCriteria($this->dataSourceName);
        $filterCriteria->addCondition('dataSourceId', $dataSourceId);
        $this->addConditionsToSuccessImport($filterCriteria);
        $filterCriteria->addOrderBy('endDate', FilterCriteria::ORDER_DESC);
        $lastImport = $this->mongoManager->findBy($filterCriteria)->getRows();
        if(count($lastImport) > 0){
            $lastImport = $lastImport[0];
            $lastImportDate = $this->formatDate($lastImport['endDate']);
        }
        return $lastImportDate;
    }

    public function getLastImportDatesByOrganization($organizationId, $dataSourceWithoutDataMining, $dataSourceIds=[]){
        $filterCriteria = new ZmartFilterCriteria($this->dataSourceName);
        $filterCriteria->addCondition('organizationId', '=', (string)$organizationId);
        if(!empty($dataSourceIds)){
            $filterCriteria->addCondition("dataSourceId",FilterCriteria::IN,$dataSourceIds);
        }
        $this->addConditionsToSuccessImport($filterCriteria);
        $filterCriteria->setGroupBy(array('dataSourceId'));
        $filterCriteria->addAggregateFunction('lastImportDate', ZmartFilterCriteria::MAX, 'endDate');
        $results = $this->mongoManager->aggregate($filterCriteria)->getRows();

        $lastImportDates = array();
        foreach ($results as $data) {
            if(isset($data['_id']) && isset($data['lastImportDate'])){
                if(in_array($data['_id'], $dataSourceWithoutDataMining)){
                    $lastImportDates[$data['_id']] = $this->utilities->getTimezonedNow()->format("d/m/Y H:i");
                } else {
                    $lastImportDates[$data['_id']] = $this->formatDate($data['lastImportDate']);
                }
            }
        }
        return $lastImportDates;
    }

    public function getLastImportDatesByDataSource($dataSourceId){
        $filterCriteria = new ZmartFilterCriteria($this->dataSourceName);
        $filterCriteria->addCondition('dataSourceId', '=', (string)$dataSourceId);
        $filterCriteria->addCondition('importStatus', '=', 'finalized');
        $this->addConditionsToSuccessImport($filterCriteria);
        $filterCriteria->setGroupBy(array('dataSourceId'));
        $filterCriteria->addAggregateFunction('lastImportDate', ZmartFilterCriteria::MAX, 'endDate');
        $results = $this->mongoManager->aggregate($filterCriteria)->getRows();

        $lastImportDates = array();
        foreach ($results as $data) {
            if(isset($data['_id']) && isset($data['lastImportDate'])){
                $lastImportDates[$data['_id']] = $this->formatDate($data['lastImportDate']);
            }
        }
        return $lastImportDates;
    }

    protected function addConditionsToSuccessImport(&$filterCriteria){
        $filterCriteria->addCondition('hasError', false);
        $filterCriteria->addCondition('endDate', '<>', null);
    }

    public function updateImports($importHistory) {
        $history['organization'] = (string)$this->sessionManager->get('organizationId');
        $persistedItems = $this->mongoManager->persist(new DataSet('importHistory', $importHistory));
    }

}