import { Http } from "@zeedhi/core";
import moment from 'moment';

/* jshint -W054 */
export class PeriodsService {
    private INVALID_PERIOD_FORMAT_MESSAGE:Error = {
		name: "Incorrect period",
		message: "Used period has an incorrect format."
	};
    //
    private PREFIX_THIS = "this";
    private PREFIX_NEXT = "next";
    private PREFIX_LAST = "last";
	private PREFIXES = [this.PREFIX_THIS, this.PREFIX_NEXT, this.PREFIX_LAST];
    //
	private SUFFIX_INCLUSIVE = "inclusive";
    private SUFFIXES = [this.SUFFIX_INCLUSIVE];
    //
	private UNIT_DAY = "day";
	private UNIT_WEEK = "week";
	private UNIT_MONTH = "month";
	private UNIT_YEAR = "year";
	private UNITS = [this.UNIT_DAY, this.UNIT_WEEK, this.UNIT_MONTH, this.UNIT_YEAR];
    //
	private PREPOSITION_IN = "in";
	private PREPOSITION_UNTIL = "until";
	private PREPOSITIONS = [this.PREPOSITION_IN, this.PREPOSITION_UNTIL];
    //
	private EXCEPTION_UNITS = {
        "days": this.UNIT_DAY,
        "weeks": this.UNIT_WEEK,
        "months": this.UNIT_MONTH,
        "years": this.UNIT_YEAR
    };
    //
    private ONE_WORD_EXPRESSIONS = {
        "today": "this-day",
        "yesterday": "last-day",
        "tomorrow": "next-day"
    };
    //
    private DATE_FORMAT = "DD/MM/YYYY";
    //
	private dataSourcePeriods = [];
    //
    private self = this;
    //
    public decodePeriod = (periodCode:any) => {
		const now = moment();
		let start = moment();
		let end = moment();
		if (periodCode) {
            if (periodCode === "last-month|until-last-day|in-last-month" && now.format('DD') === '01') {
                periodCode = "until-last-day|in-last-month";
            }
			const periodParams = this.getPeriodParams(periodCode);
			const buildInterval = this.buildInterval;
			periodParams.forEach(function (params:any) {
				const period = buildInterval(params, start, end);
				start = period.start;
				end = period.end;
			});
		}
		const interval = this.factoryInterval(start, end);
		return interval;
	};
    //
    private getPeriodParams = (periodCode:any) => {
		const periodParams:any = [];
		const expressions = periodCode.split("|");
        const decodeExpression = this.decodeExpression;
		expressions.forEach(function (expression:any) {
			const param = decodeExpression(expression);
			periodParams.push(param);
		});
		return periodParams;
	};
    //
    private decodeExpression = (expression:any) => {
		const splitedExpression:any = [];
        const ONE_WORD_EXPRESSIONS = this.ONE_WORD_EXPRESSIONS;
		expression.split("-").forEach(function (expressionPart: keyof typeof ONE_WORD_EXPRESSIONS) {
			const expressPart = expressionPart in ONE_WORD_EXPRESSIONS ? ONE_WORD_EXPRESSIONS[expressionPart] : expressionPart;
			const expressPartSplited = expressPart.split("-");

			expressPartSplited.forEach(function (part:any) {
				splitedExpression.push(part);
			});
		});
		const decodedExpression = this.recognizeExpressionTerms(splitedExpression);
		return decodedExpression;
	};
    //
	private recognizeExpressionTerms = (splitedExpression:any) => {
		const decodedExpression:any = {};
        const EXCEPTION_UNITS = this.EXCEPTION_UNITS;
        const PREPOSITIONS = this.PREPOSITIONS;
        const PREFIXES = this.PREFIXES;
        const UNITS = this.UNITS;
        const SUFFIXES = this.SUFFIXES;
        const INVALID_PERIOD_FORMAT_MESSAGE = this.INVALID_PERIOD_FORMAT_MESSAGE;
		splitedExpression.forEach(function (termParam: keyof typeof EXCEPTION_UNITS) {
            let term:any = termParam;
			if (EXCEPTION_UNITS[termParam]) {
				term = EXCEPTION_UNITS[termParam];
			}
			if (PREPOSITIONS.indexOf(term) !== -1) {
				decodedExpression.preposition = term;
			} else if (PREFIXES.indexOf(term) !== -1) {
				decodedExpression.prefix = term;
			} else if (UNITS.indexOf(term) !== -1) {
				decodedExpression.unit = term;
			} else if (!isNaN(parseInt(term, 10))) {
				decodedExpression.amount = term;
			} else if (SUFFIXES.indexOf(term) !== -1) {
				decodedExpression.suffix = term;
			} else {
				throw INVALID_PERIOD_FORMAT_MESSAGE;
			}
		});
		return decodedExpression;
	};
    //
    public conditionPeriod = (periodString:any, isCompare:any) => {
        if (periodString.split("if (", 1)[0] === "") {
			let retorno = '{';
			let dot = ';';
			if (!periodString.split("return")[1]) {
				retorno = '{ return "';
				dot = '";';
			}
            let periodTest = periodString.split('{');
            let period = "";
            for (let i = 0; i < periodTest.length - 1; i += 1) {
                period = period + periodTest[i] + retorno;
            }
            period += periodTest[periodTest.length - 1];
            periodTest = period.split(';');
            period = "";
            for (let i = 0; i < periodTest.length - 1; i += 1) {
                period = period + periodTest[i] + dot;
            }
            period += periodTest[periodTest.length - 1];
            if (isCompare === true) {
                return period;
            }
			/* eslint-disable */
			const condition = new Function(period);
			/* eslint-enable */
			return condition();
        }
		return periodString;
    };
	//
	private buildInterval = (params:any, start:any, end:any) => {
		const preposition = params.preposition || null;
		const prefix = params.prefix || null;
		const amount = params.amount || 1;
		const unit = params.unit || null;
		const suffix = params.suffix || null;
		switch (prefix) {
			case this.PREFIX_LAST:
				return this.buildIntervalToLastPrefix(preposition, amount, unit, suffix, start, end);
			case this.PREFIX_NEXT:
				return this.buildIntervalToNextPrefix(preposition, amount, unit, suffix, start, end);
			case this.PREFIX_THIS:
				return this.buildIntervalToThisPrefix(preposition, unit, suffix, start, end);
			default:
				return this.buildIntervalToNoPrefix(preposition, amount, unit, suffix, start, end);
		}
	};
	//
	private buildIntervalToLastPrefix = (preposition:any, amount:any, unit:any, suffix:any, periodStart:any, periodEnd:any) => {
		const endAmount = suffix === this.SUFFIX_INCLUSIVE ? 0 : 1;
		amount = suffix === this.SUFFIX_INCLUSIVE ? amount - 1 : amount;
		const start = moment().subtract(amount, unit);
		const end = moment().subtract(endAmount, unit);
		if (preposition === this.PREPOSITION_IN) {
			periodStart = this.getMomentWhenInClause(periodStart, unit, start);
			periodEnd = this.getMomentWhenInClause(periodEnd, unit, end);
		} else {
			periodEnd = end.endOf(unit);

			if (!preposition) {
				periodStart = start.startOf(unit);
			}
		}
        if (periodStart > periodEnd) {
            periodEnd = this.copyDate(periodStart);
        }

		return {
			start: periodStart,
			end: periodEnd
		};
	};
	//
    private copyDate = (data:any) => moment(data.format("YYYY-MM-DD"), "YYYY-MM-DD");
	//
	private buildIntervalToNextPrefix = (preposition:any, amount:any, unit:any, suffix:any, periodStart:any, periodEnd:any) => {
		const startAmount = suffix === this.SUFFIX_INCLUSIVE ? 0 : 1;
		const endAmount = suffix === this.SUFFIX_INCLUSIVE ? amount - 1 : amount;
		const start = moment().add(startAmount, unit);
		const end = moment().add(endAmount, unit);
		if (!preposition) {
			periodStart = start.startOf(unit);
			periodEnd = end.endOf(unit);
		} else if (preposition === this.PREPOSITION_UNTIL) {
			periodEnd = end.startOf(unit);
		} else if (preposition === this.PREPOSITION_IN) {
			periodStart = this.getMomentWhenInClause(periodStart, unit, start);
			periodEnd = this.getMomentWhenInClause(periodEnd, unit, end);
		}
		return {
			start: periodStart,
			end: periodEnd
		};
	};
	//
	private buildIntervalToThisPrefix = (preposition:any, unit:any, suffix:any, periodStart:any, periodEnd:any) => {
		const start = moment();
		const end = moment();
		if (!preposition) {
			periodStart = start.startOf(unit);
			periodEnd = end.endOf(unit);
		} else if (preposition === this.PREPOSITION_UNTIL) {
			if (suffix !== this.SUFFIX_INCLUSIVE) {
				end.subtract(1, unit);
			}
			periodEnd = end.endOf(unit);
		} else if (preposition === this.PREPOSITION_IN) {
			periodStart = this.getMomentWhenInClause(periodStart, unit, start);
			periodEnd = this.getMomentWhenInClause(periodEnd, unit, end);
		}
        if (periodStart > periodEnd) {
            periodEnd = this.copyDate(periodStart);
        }
		return {
			start: periodStart,
			end: periodEnd
		};
	};
	//
	private buildIntervalToNoPrefix = (preposition:any, amount:any, unit:any, suffix:any, periodStart:any, periodEnd:any) => {
		if (!preposition) {
			periodStart = this.setDateParam(unit, amount, periodStart);
		} else if (preposition === this.PREPOSITION_UNTIL) {
			periodEnd = this.setDateParam(unit, amount, periodEnd);
		}
		return {
			start: periodStart,
			end: periodEnd
		};
	};
	//
	private setDateParam = (unit:any, amount:any, momentLocal:any) => {
		switch (unit) {
			case this.UNIT_DAY:
				return momentLocal.date(amount);
			case this.UNIT_WEEK:
				return momentLocal.week(amount);
			case this.UNIT_MONTH:
				return momentLocal.month(amount);
			case this.UNIT_YEAR:
				return momentLocal.year(amount);
			default:
		}
		return moment();
	};
	//
	private getMomentWhenInClause = (initialMoment:any, unit:any, newMoment:any) => {
		const periodDate = initialMoment._d;
		const newDate = newMoment._d;
		this.getNewDateFromParams(unit, periodDate, newDate);
		return moment(periodDate);
	};
	//
	private getNewDateFromParams = (unit:any, periodDate:any, newDate:any) => {
		if (unit === this.UNIT_DAY) {
			periodDate = newDate;
		} else if (unit === this.UNIT_MONTH) {
			periodDate.setMonth(newDate.getMonth());
			periodDate.setFullYear(newDate.getFullYear());
		} else if (unit === this.UNIT_YEAR) {
			periodDate.setFullYear(newDate.getFullYear());
		}
	};
	//
	private factoryInterval = (start:any, end:any) => {
		if (!start) {
			start = moment();
		}
		if (!end) {
			end = moment();
		}
		return {
			"start": start.format(this.DATE_FORMAT),
			"end": end.format(this.DATE_FORMAT),
			"customized": true
		};
	};
    //
	// Getting Periods Informations
	//
	private getPeriodValueByCodedName = (periodCode:any) => periodCode.split("'")[1] || periodCode;
	//
	private getCustomPeriodCode = (periodString:any) => {
		let periodCode = '';
		if (periodString) {
			const interval = this.getPeriodMathExpression(periodString);
			const condition = this.conditionPeriod(periodString, true);
			if (interval && interval._id) {
				periodCode = this.getFunctionPeriodMathExpression(periodString);
			} else if (periodString.split("if (", 1)[0] !== "") {
				periodCode = "@PeriodsUtilities.decodePeriod('" + periodString + "')@";
            } else {
				periodCode = "@PeriodsUtilities.decodeCondition('" + condition + "')@";
            }
		}
		return periodCode;
	};
	//
	private isHexadecimal = (str:any) => {
		const regexp = /^[0-9a-fA-F]+$/;
		if (regexp.test(str)) {
			return true;
		}
		return false;
	};
	//
	private getPeriodFromCodeAndPeriods = (periodCode:any, intervals:any) => {
		let period;
		const conditionPeriod = this.conditionPeriod;
		if (intervals && intervals.length) {
			intervals.forEach(function (interval:any) {
				if (interval.periodsCode.split("if (", 1)[0] === "") {
					if (conditionPeriod(interval.periodsCode, true) === periodCode) {
						period = interval;
					}
				} else if (interval.periodsCode === periodCode) {
					period = interval;
				}
				if (periodCode === interval._id) {
					period = interval;
				}
			});
		}
		return period;
	};
	//
	private validatePeriodIntervals = (period:any) => {
		if (period && period.start && period.end) {
			return true;
		}
		return false;
	};
	//
	private getPeriodLabelFromIntervals = (intervals:any, periodCode:any) => {
		const period = intervals.filter(function (interval:any) {
			return interval.periodsCode === periodCode;
		}).shift();
		return period ? period.name : null;
	};
	//
	private removeCustomLabel = (periodField:any) => {
		periodField.label = periodField.label.split(' ')[0];
	};
	//
    private decodeCondition = (period:any) => {
        if (period.split("if (", 1)[0] === "") {
            const condition = (periodLocal:any) => {};
            return this.decodePeriod(condition(''));
        }
        return null;
    };
	//
	private getPeriodMathExpression = (_id:any) => {
		let interval:any = {};
		this.dataSourcePeriods.some(function (period:any) {
			if (period._id === _id && !period.Erro) {
				interval = period.interval;
				interval.label = period.name;
				return true;
			}
			return false;
		});
		return interval;
	};
	//
	private getFunctionPeriodMathExpression = (_id:any) => "@PeriodsUtilities.getPeriodMathExpression('" + _id + "')@";
	//
	// Date informations
	//
    public getDateByExpressions = async (expressions: any) => {
		const data = { "requestType": "Row", "row": { "datesExpression": expressions}};
        return Http.post("/getDateByDateExpression", data).then(function (response) {
			return response.data;
		});
	};
}
